use crate::{decks::new_blackjack_shoe, hand::Hand};

pub struct BlackjackGame {
    shoe: Hand,
    players: Vec<Player>,
    dealer_hand: Hand,
    state: GameState,
}

pub struct Player {
    hand: Hand,
    state: PlayerState,
}

impl Player {
    fn new() -> Self {
        Player {
            hand: Hand::new(),
            state: PlayerState::Playing,
        }
    }

    pub fn get_hand(&self) -> &Hand {
        &self.hand
    }

    pub fn get_state(&self) -> &PlayerState {
        &self.state
    }
}

#[derive(Clone, Copy, Debug, PartialEq)]
pub enum PlayerState {
    Playing,
    Standing,
    DoubleDown,
    Busted,
    Blackjack,
    Maxed, // Reached 21
}

#[derive(Clone, Copy, Debug)]
pub enum PlayMoves {
    Hit,
    Stand,
    DoubleDown,
    Deal(usize),
}

pub enum GameState {
    Starting,
    Over,              // Game is over
    PlayerTurn(usize), // Its the turn of the player
}

impl BlackjackGame {
    pub fn new() -> Self {
        BlackjackGame {
            shoe: new_blackjack_shoe(6),
            players: Vec::new(),
            dealer_hand: Hand::new(),
            state: GameState::Starting,
        }
    }

    pub fn get_dealer_hand(&self) -> &Hand {
        &self.dealer_hand
    }

    pub fn get_state(&self) -> &GameState {
        &self.state
    }

    pub fn get_player(&self, index: usize) -> Option<&Player> {
        self.players.get(index)
    }

    pub fn get_player_count(&self) -> usize {
        self.players.len()
    }

    pub fn play(&mut self, action: PlayMoves) -> bool {
        match (&self.state, action) {
            (GameState::PlayerTurn(i), PlayMoves::Hit) => {
                let Some(player) = self.players.get_mut(*i) else {
                    return false;
                };

                if player.state != PlayerState::Playing {
                    return false;
                }

                player.hand.add_card(self.shoe.pop_card().unwrap());
                match player.hand.get_blackjack_value().cmp(&21) {
                    std::cmp::Ordering::Equal => {
                        player.state = PlayerState::Maxed;
                    }
                    std::cmp::Ordering::Greater => {
                        player.state = PlayerState::Busted;
                    }
                    std::cmp::Ordering::Less => {
                        // Player is still playing
                    }
                };
            }
            (GameState::PlayerTurn(i), PlayMoves::DoubleDown) => {
                let Some(player) = self.players.get_mut(*i) else {
                    return false;
                };

                if player.state != PlayerState::Playing {
                    return false;
                }

                player.hand.add_card(self.shoe.pop_card().unwrap());
                match player.hand.get_blackjack_value().cmp(&21) {
                    std::cmp::Ordering::Equal | std::cmp::Ordering::Less => {
                        player.state = PlayerState::Maxed;
                    }
                    std::cmp::Ordering::Greater => {
                        player.state = PlayerState::Busted;
                    }
                };
            }
            (GameState::PlayerTurn(i), PlayMoves::Stand) => {
                let Some(player) = self.players.get_mut(*i) else {
                    return false;
                };

                if player.state != PlayerState::Playing {
                    return false;
                }

                player.state = PlayerState::Standing;
            }
            (GameState::Over, PlayMoves::Deal(player_count))
            | (GameState::Starting, PlayMoves::Deal(player_count)) => {
                for i in 0..player_count {
                    self.players.push(Player::new());
                    self.players[i].hand.add_card(self.shoe.pop_card().unwrap());
                }

                self.dealer_hand.add_card(self.shoe.pop_card().unwrap());

                for i in 0..player_count {
                    self.players[i].hand.add_card(self.shoe.pop_card().unwrap());

                    if self.players[i].hand.is_backjack() {
                        self.players[i].state = PlayerState::Blackjack;
                    }
                }

                self.dealer_hand.add_card(self.shoe.pop_card().unwrap());

                self.state = GameState::PlayerTurn(0);
            }
            (_, PlayMoves::Deal(_)) | (GameState::Over, _) | (GameState::Starting, _) => {
                return false;
            }
        }

        let player_count = self.players.len();

        match self.state {
            GameState::PlayerTurn(i) => {
                if self.players[i].state == PlayerState::Playing {
                    self.state = GameState::PlayerTurn(i);
                } else if i + 1 >= player_count {
                    loop {
                        let dealer_value = self.dealer_hand.get_blackjack_value();
                        if dealer_value >= 17 {
                            // Check winners
                            self.state = GameState::Over;
                            return true;
                        }

                        self.dealer_hand.add_card(self.shoe.pop_card().unwrap());

                        if self.dealer_hand.get_blackjack_value() > 21 {
                            // Dealer busts
                            self.state = GameState::Over;
                            return true;
                        }
                    }
                } else {
                    self.state = GameState::PlayerTurn(i + 1);
                }
                return true;
            }
            _ => {
                return false;
            }
        }
    }
}
